#assignRemainingPorts.awk
#
#This script takes a list of device names of ports we expect to find on the 
# motherboard, and walks the pciconf results looking for devices that have not
# been assinged to the physical ports.  The first argNumMoboPorts of the 
# physical/logical port mapping array (argMbPort) consist of offset numbers
# indicating the order of the the interface devices found (see SDD for details).
# The remaining elements in the array are the device names of the physical
# ports in each of the slots.  This script replaces the offset numbers of the
# first argNumMoboPorts elements with the device names of the port, completing
# the mapping.

#Set our field separator to " " (default)
BEGIN {
    FS = " "
    numMoboDevsFound = 0
    firstTime = 1
}   
#Set our output variables (the missing array elements)
END {
    for (i=1; i<=argNumMoboPorts; ++i) 
    {
        mbPort[i] = moboDev[aMbPort[i]]
        printf( "mbPort[%s]=\"%s\"\n", i, mbPort[i] )
    }
}
#Given an array, a starting index, a range to test, and a test string, return 1
# if the string is the exact value of one of the slots and 0 otherwise
    function ArrayContains ( array, start, size, text )
    {
        for( i=start; i<(size+start); ++i )
        {
            if( array[i] == text )
            {
                return 1
            }
        }
        return 0
    }
    #____________________________________________________________________
    # This script parses the pciconf input looking for interface devices that
    # are not listed in the current port mappings and assigns them accordingly.
    # Inputs:
    #   argNumMoboPorts - number of ports on the motherboard
    #   argNumExpPorts - number of ports assigned to pci expansion slots
    #   argMbPort - array (newline separated values) of physical ports.  When
    #     this script is called, the first <argNumMoboPorts> of them will
    #     correspond to the order in which to assign the non-expansion slot
    #     devices (this is what we're "filling in").  The remaining values
    #     will be the actual device names of the already assigned devices from
    #     expansion slots.  We use these to determine which devices remain.
    #   argNumNames - number of names in the following array
    #   argPortNames - array (newline separated values) of the device names
    #     (minus the number portion) of the interface devices we expect to 
    #     find on the motherboard.
    #
    # Outputs:
    #   mbPort[1-argNumMoboPorts] - this script will set mbPort[1] through
    #     mbPort[argNumMoboPorts] with the name of the device to which the 
    #     given port maps.
    #____________________________________________________________________
    {
        #Script arguments are not available until the body of the script, 
        # so we need a first time flag so we can parse our list of array
        # values into an array.
        if (firstTime) 
        {
            firstTime = 0
            
            #arrays were passed in as newline separated value tags...parse'em
            split ( argMbPort, argMbPortVal, "\n" )
            split ( argPortNames, argPortNamesVal, "\n" )
            for (i=1; i<=argNumMoboPorts+argNumExpPorts; ++i) 
            {
                aMbPort[i] = argMbPortVal[i+1]
            }
            for (i=1; i<=argNumNames; ++i) 
            {
                aPortNames[i] = argPortNamesVal[i+1]
            }
        }

        #look for things like em0@pci1:2:0
        #                     ^^^^
        pattern = "[A-z]+[0-9]+@"
        #if this line contains one as the first field
        if ( match ($1, pattern) )
        {
            #pull out the value (minus the @)
            dev = substr($1, RSTART, RLENGTH-1)
            #if this device is not in the portion of ports assigned to
            # expansion slots
            if (!ArrayContains( aMbPort, argNumMoboPorts+1, argNumExpPorts, dev )) 
            {
                #match the name only portion
                if ( match ( dev, /[A-z]+/ ) ) 
                {
                    #extract the name only portion
                    name = substr($1, RSTART, RLENGTH)
                    
                    #Make sure our names array contains this device name
                    if (ArrayContains( aPortNames, 1, argNumNames, name )) 
                    {
                        #This device's name was in the list of mobo port names
                        # Assign the port and increment our count
                        numMoboDevsFound++
                        moboDev[numMoboDevsFound] = dev
                    }
                }
            }
        } 
    } 

