#extractCardCfgInfo.awk
#
#This script parses the configuration file for a card and extracts all of the 
# "config." and "misc." tags.  Note when this is called, the config file must
# be known to match the card.

#Set our field separator to "="
BEGIN {
    FS = "="
    crdModel="none specified" 
    crdNumPorts=0
    crdNumMisc=0
    portsCounted=0
    crdNameMapping=""

    #set our success code to 0 (error) in case the script dies for some reason
    printf( "res=0\n" )
    logPrefix = "mobomap: "
    res = 0
}

#Set our output variables
END { 
    #TODO: deal with " in the strings
    printf( "crdModel=\"%s\"\n", crdModel ) 
    
    printf( "crdNumPorts=\"%s\"\n", crdNumPorts ) 
    for (i=1; i<=crdNumPorts; ++i)
    {
        if (crdPort[i] == "") 
        {
            #default unspecified ports to their number
            crdPort[i] = i
            #need to increment since defaulted ports count as a port found
            ++portsCounted
        }
        #Check each port for uniqueness (TODO: No longer needed?
#        if( uniquePorts[crdPort[i]] == 1 )
#        {
#            system ("logger -p local0.error \"" logPrefix "(" argFileName ") Duplicate reference to card port (card.ports.sel" i "=" crdPort[i] ")\"")
#            res = 2
#        }
#        else
#        {
#            uniquePorts[crdPort[i]] = 1 
#        }

        if (argReversed) 
        {
            printf( "crdPort[%s]=\"%s\"\n", i, crdPort[crdNumPorts-i+1] ) 
        }
        else
        {
            printf( "crdPort[%s]=\"%s\"\n", i, crdPort[i] ) 
        }
    }
    printf( "crdNameMapping=\"%s\"\n", crdNameMapping );

    if (portsCounted > crdNumPorts) 
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Too many ports specified ("portsCounted"); config specifies "crdNumPorts"\"")
    }

    printf( "crdNumMisc=\"%s\"\n", crdNumMisc ) 
    for (i=0; i<crdNumMisc; ++i)
    {
        printf( "crdMisc[%s]=\"%s\"\n", i, crdMisc[i] ) 
    }
    
    #set our return code 
    if (res == 2) 
    {
        printf( "res=2\n" )
    }
    else
    {
        printf( "res=1\n" )
    }
}                    

function CheckForDupes (field1, field2)
{
    if (tags[field1] == 1) 
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Duplicate tag found (" field1 "=" field2 ")\"")
    }
    tags[field1]=1
}
#____________________________________________________________________
# This script expects the following input parameters:
#  argReversed - 1 if the card that this slot is in is reversed, 0 otherwise.
#  argFileName - Name of the file being parsed
#
# This script parses specified config file and sets the following
# variables based on the configuration tags found:
#   crdModel - string of all of the cards described by this file
#   crdNumPorts - number of ports on the card
#   crdPort[] - array of indices relating to the order in which pci devices
#               are discovered (used to map devices names to physical ports
#   crdMiscTags[] - array of miscellaneous tag/value pairs
#   crdNameMapping - name to use for devices if not the same as pci dev name
#   res - 0 if the script doesn't execute to completion (error), 1 if it
#         executed successfully, 2 if executed to completion, but there
#         was an error that was logged to syslog
#____________________________________________________________________
{
    #if it is not a comment line
    if (match ($1, /^[ \t]*#/) == 0)
    {
        #if it is not a "match." type tag
        if (match ($1, /^[ \t]*match\./) == 0)
        {
            #Normalize the fields.  We expect a tag containing no '=',
            # followed by any value string.  Since the value could contain 
            # an '=' and we have to check that there are only 2 fields
            # found in this record, and if there are more, concatenate
            # $2 with the additional ones (inserting a '=')
            if ( (length ($1) != 0) && (length ($2) == 0)) 
            {
                system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Tag missing value (" $1 "); ignoring it\"")
            }
            else
            {
                field2 = $2
                if ( NF > 2 )
                {
                    for (i=3; i<=NF; ++i)
                    {
                       field2 = field2 "=" $i
                    }
                }

                #convert " to \"
                pattern = "\""
                if (match (field2, pattern))
                {
                    str1 = substr( field2, 1, RSTART-1 )
                    str2 = substr( field2, RSTART+1 )
                    field2 = str1 "\\\""
                    while (match (str2, pattern))
                    {
                        str1 = substr( str2, 1, RSTART-1 )
                        str2 = substr( str2, RSTART+1 )
                        field2 = field2 str1 "\\\""
                    }
                    field2 = field2 str2 
                }

                #strip off trailing newline
                if (match (field2, /\n$/)) 
                {
                    len = length ( field2 ) - 1
                    field2 = substr( field2, 1, len )
                }

                #Now parse the tags
                if ( match ($1, /^[ \t]*card.model/) )
                {
                    CheckForDupes( $1, field2 )
                    #it is a model string tag
                    if( crdModel == "none specified" )
                    {
                        crdModel = field2
                    }
                    else
                    {
                        crdModel = crdModel " or " field2
                    }
                }
                else if ( match ($1, /^[ \t]*card.ports/) )
                {
                    #It's a ports tag
                    if ( match ($1, /^[ \t]*card.ports.count/) )
                    {
                        CheckForDupes( $1, field2 )
                        crdNumPorts = field2
                    }
                    else if ( offset = match ($1, /^[ \t]*card.ports.sel/) )
                    {
                        CheckForDupes( $1, field2 )
                        offset = offset + RLENGTH
                        len = length ( $1 ) - offset + 1
                        selnum = substr ( $1, offset, len )
                        #test uniqueness
                        if ( crdPort[field2] == "" )
                        {
                            crdPort[field2] = selnum
                        }
                        else
                        {
                            system ("logger -p local0.error \"" logPrefix "(" argFileName ") Duplicate reference to card port (card.ports.sel" selnum "=" field2 ")\"")
                            res = 2
                        }
                    }
                    else if ( match ($1, /^[ \t]*card.ports.name[ \t]*$/) )
                    {
                        crdNameMapping = field2
                    }
                    else
                    {
                        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Unrecognized tag found (" $1 ")\"")
                    }
                }
                else if ( match ($1, /^[ \t]*misc[ \t]*$/) )
                {
                    #It's a misc tag
                    crdMisc[crdNumMisc] = field2
                    ++crdNumMisc
                }
                else if ( match ($1, /^[ \t]*$/ ) )
                {
                    #ignore blank lines
                }
                else 
                {
                    #anything else is unrecognized
                    system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Unrecognized tag found (" $1 ")\"")
                }
            }
        }
    }
}


