#extractMoboCfgInfo.awk
#
#This script parses the configuration file for a motherboard and extracts all
# of the "config." and "misc." tags.  Note when this is called, the config
# file must be known to match the motherboard.

#Set our field separator to "="
BEGIN {
    expNumRenameIf=0
    expNumMisc=0
    expNumHideIf=0
    FS = "="
    res = 1
    logPrefix = "expansion: " 
    #set our success code to 0 (error) in case the script dies for some reason
    printf( "res=0\n" )
}

#Set our output variables
END { 
    printf( "expNumRenameIf=\"%s\"\n", expNumRenameIf )
    for (i=0; i<expNumRenameIf; ++i)
    {
        printf( "expRenameIf[%s]=\"%s\"\n", i, expRenameIf[i] )
    }

    printf( "expNumMisc=\"%s\"\n", expNumMisc )
    for (i=0; i<expNumMisc; ++i)
    {
        printf( "expMisc[%s]=\"%s\"\n", i, expMisc[i] )
    }

    printf( "expNumHideIf=\"%s\"\n", expNumHideIf )
    for (i=0; i<expNumHideIf; ++i)
    {
        printf( "expHideIf[%s]=\"%s\"\n", i, expHideIf[i] )
    }

    #set our success code to 1
    printf( "res=\"" res "\"\n" )
}                    

function CheckForDupes( field1, field2 )
{
    if (tags[field1] == 1) 
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Duplicate tag found (" field1 "=" field2 ")\"")
    }
    tags[field1]=1
    return 0
}

#____________________________________________________________________
# This script expects the following input parameters:
#   argFileName - Name of the file being parsed
#
# This script parses specified config file and sets the following
# variables based on the configuration tags found:
#   expRenameIf: ports on expansion card need to be renamed
#   expHdeIf: port on expansion card need to be hide
#   expMisc:    port names 
#   res - 0 if the script doesn't execute to completion (error), 1 if it
#         executed successfully, 2 if executed to completion, but there
#         was an error that was logged to syslog
#____________________________________________________________________
{
    #See SDD for tag syntax
    #if it is not a comment line
    if (match ($1, /^[ \t]*#/) == 0)
    {
        #if it is not a "match." type tag
        if (match ($1, /^[ \t]*match\./) == 0)
        {
            #Normalize the fields.  We expect a tag containing no '=',
            # followed by any value string.  Since the value could contain 
            # an '=' and we have to check that there are only 2 fields
            # found in this record, and if there are more, concatenate
            # $2 with the additional ones (inserting a '=')
            if ( (length ($1) != 0) && (length ($2) == 0)) 
            {
                system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Tag missing value (" $1 "); ignoring it\"")
            }
            else
            {
                field2 = $2
                if ( NF > 2 )
                {
                    for (i=3; i<=NF; ++i)
                    {
                       field2 = field2 "=" $i
                    }
                }

                #convert " to \"
                pattern = "\""
                if (match (field2, pattern))
                {
                    str1 = substr( field2, 1, RSTART-1 )
                    str2 = substr( field2, RSTART+1 )
                    field2 = str1 "\\\""
                    while (match (str2, pattern))
                    {
                        str1 = substr( str2, 1, RSTART-1 )
                        str2 = substr( str2, RSTART+1 )
                        field2 = field2 str1 "\\\""
                    }
                    field2 = field2 str2 
                }

                #strip off the trailing newline
                if (match (field2, /\n$/)) 
                {
                    len = length ( field2 ) - 1
                    field2 = substr( field2, 1, len )
                }

                #Now parse the tags
                if ( match ($1, /^[ \t]*renameif[ \t]*$/) )
                {
                    expRenameIf[expNumRenameIf] = field2
                    ++expNumRenameIf
                }
                else if ( match ($1, /^[ \t]*hideif[ \t]*$/) )
                {
                    expHideIf[expNumHideIf] = field2
                    ++expNumHideIf
                }

                else if ( match ($1, /^[ \t]*misc[ \t]*$/) )
                {
                    expMisc[expNumMisc] = field2
                    ++expNumMisc
                }
                else if ( match ($1, /^[ \t]*$/ ) ) 
                {
                    #ignore blank lines
                }
                else 
                {
                    #anything else is unrecognized
                    system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Unrecognized tag found (" $1 ")\"")
                }
            }
        }
    }
}


