#extractPciconfForDevices.awk
#
#This script parses the pciconf results and returns the configuration for the
# subset of the devices that were found in the incoming array of device names.
# The remaining config (the stuff that didn't match the devices) is also
# returned.

#Set the field separator to the newline
BEGIN {
    FS = "\n"
    firstTime = 1
    #Every time we hit a device we'll decide whether it matches one of the
    # devices passed in.  If does, we'll set this flag, and write every line 
    # until the next device to the pciconfForThisSlot var.
    matching = 0
    pciconfForThisSlot = ""
    pciconfRemaining = ""
    linesForThisSlot = 0
    linesRemaining = 0
}             

#Set our output variables
END {
     printf( "pciconfForThisSlot=\"%s\"\n", pciconfForThisSlot ) 
     printf( "pciconfRemaining=\"%s\"\n", pciconfRemaining ) 
}                    

#Given an array, a starting index, a range to test, and a test string, return 1
# if the string is the exact value of one of the slots and 0 otherwise
    function ArrayContains ( array, start, size, text )
    {
        for( i=start; i<(size+start); ++i )
        {
            if( array[i] == text )
            {
                return 1
            }
        }
        return 0
    }
#____________________________________________________________________
# Inputs is:
#   argSlotDevs - An array of devices in this slot
#   argNumDevs - The number of devices in the array above
#
# It sets the following variables:
#   pciconfForThisSlot - array of lines from the input pciconf results that
#                        belong to this slot
#   pciconfRemaining - array of lines from the input pciconf results that do 
#                      not belong to this slot
#____________________________________________________________________
    {
        #Script arguments are not available until the body of the script, 
        # so we need a first time flag so we can get at the values
        if (firstTime) 
        {
            firstTime = 0
            split ( argSlotDevs, argSlotDevsVal, "\n" )
            for (i=1; i<=argNumDevs+1; ++i) 
            {
                aSlotDevs[i] = argSlotDevsVal[i+1]
            }
        }
        if ( match ($1, /[A-z]+[0-9]+@pci[0-9]+:[0-9]+:[0-9]+/) ) 
        {
            match ($1, /[A-z]+[0-9]+/)
            dev = substr( $1, RSTART, RLENGTH )
            if (ArrayContains(aSlotDevs, 1, argNumDevs, dev)) 
            {
                matching = 1

            }
            else
            {
                matching = 0
            }
        }
        
        if (matching) 
        {
            if (length(pciconfForThisSlot)) 
            {
                pciconfForThisSlot = pciconfForThisSlot "\n" $1
            }
            else
            {
                pciconfForThisSlot = $1
            }
            ++linesForThisSlot
        }
        else
        {
            if (length(pciconfRemaining)) 
            {
                pciconfRemaining = pciconfRemaining "\n" $1
            }
            else
            {
                pciconfRemaining = $1
            }
            ++linesRemaining
        }
    } 


