#getDevicesOnPciSelect.awk
#
#This script parses the pciconf results and returns the devices found at the
# specified PCI selector

#Set the field separator to the newline
BEGIN {
    FS = "\n"
    firstTime = 1
}             

#Set our output variables
END {
    printf( "mbNumDevs=\"%s\"\n", mbNumDevs ) 
    for (i=0; i<mbNumDevs; ++i)
    {
        printf( "mbIdx[%s]=\"%s\"\n", i, mbIdx[i] ) 
        printf( "mbDevs[%s]=\"%s\"\n", i, mbDevs[i] ) 
        printf( "mbBridge[%s]=\"%s\"\n", i, mbBridge[i] ) 
        printf( "mbBusses[%s]=\"%s\"\n", i, mbBusses[i] ) 
        printf( "mbDevices[%s]=\"%s\"\n", i, mbDevices[i] ) 
        printf( "mbFns[%s]=\"%s\"\n", i, mbFns[i] ) 
    }
    printf( "mbNumCardBriges=\"%s\"\n", mbNumCardBridges ) 
}                    
#____________________________________________________________________
# This script parses the pciconf input and determines the identity of the
# device at the specified PCI select.  Inputs are:
#   argPciSel - The selector to use.  Depending on argBusOnly, this selector has
#               the form pci<bus#> (when argBusOnly is 1) or 
#               pci<bus#>:<device#> (when argBusOnly is 0)
#   argStartCount - Starting offset to start adding devices we find to the array 
#   argBusOnly - Indicates form of the argPciSel
#   mbNumCardBridges - Number of bridges found so far (only passed in when
#                      we're using argBusOnly)
#
# It sets the following variables:
#   mbNumDevs - number of devices found on the specified PCI select
#   mbIdx[] - array of indices for shell 
#   mbDevs[] - array of device names at this select
#   mbBridge[] - array of bools with offsets that match devices - indicates
#                wether the corresponding device is a bridge
#   mbSecBus[] - array of ints with offsets that match devices - filled in
#                only if the device is a bridge, this value gives us the 
#                bridge's secondary bus number
#   mbBusses[] - array of bus numbers with offsets that match devices
#   mbDevices[] - array of device numbers with offsets that match devices
#   mbFns[] - array of function numbers with offsets that match devices
#   mbNumCardBridges - total number of bridges on cards (var is passed in,
#                      possibly incremented, and returned)
#
# In the event that a bridge is discovered at the select, the devices
# behind the bridge are discovered and populated.
#____________________________________________________________________
    {
        #Script arguments are not available until the body of the script, 
        # so we need a first time flag so we can get at the values
        if (firstTime) 
        {
            mbNumDevs = argStartCount
            firstTime = 0
        }
        #if the bus only flag was set, our argPciSel looks like pci2, otherwise
        # it looks like pci2:3
        if (argBusOnly == 1) 
        {
            pattern = "[A-z]+[0-9]+@" argPciSel ":[0-9]+:[0-9]+"
        }
        else
        {
            pattern = "[A-z]+[0-9]+@" argPciSel ":[0-9]+"
        }
        #Find the pattern
        if ( offset = match ($1, pattern) )
        {
            #This device matches the pci selector, split it at the @
            split ( substr($1, offset, RLENGTH ), split1, "@" )
            #Split everything after the @ using the : as the separator
            split ( split1[2], split2, ":" )
            #This converts something like em2@pci2:3:0 to {em2 {2 3 0}}
            #    or split1[]={em2 split2}   split2[]={2 3 0}
            mbIdx[mbNumDevs] = mbNumDevs
            mbDevs[mbNumDevs] = split1[1]
            mbBusses[mbNumDevs] = substr( split2[1], length("pci") + 1 )
            mbDevices[mbNumDevs] = split2[2]
            mbFns[mbNumDevs] = split2[3]
            mbBridge[mbNumDevs] = 0
            mbSecBus[mbNumDevs] = 0
            
            #Find the header type tag
            offset = match ($1, /[ \t]+hdr=0x0/)
            if (offset != 0) \
            {
                #if's a type 1 header (PCI-PCI bridge)
                offset = offset + RLENGTH
                if ( substr( $1, offset, 1 ) == "1" ) 
                {
                    #it's header type 1 (PCI-PCI bridge)
                    mbBridge[mbNumDevs] = 1
                    ++mbNumCardBridges
                }
            }
            mbNumDevs++
        } 
        
    } 


