#matchCardToCfgFile.awk
#
#This script parses the specified configuration file and determines whether
# the card is a match.

#Field separator is "="
#Start by setting the return var to 0. This way if there's an error in the
# script and we don't end up running our END section, the caller will know.
BEGIN {
    print "expansionCfgFileMatch=0"
    matched = 1
    FS = "="
    logPrefix = "expansionmap: "

}
#Set our output variables
END {
    if (matchTagsRecognized != 0) 
    {
        if ( matched == 1 )
        {
            print "expansionCfgFileMatch=1"
        }
        else
        {
            print "expansionCfgFileMatch=2"
            printf ("failedTag=\"%s\"\n", failedTag)
        }
    }
    else
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") No match tags specified; ignoring file\"")
        print "expansionCfgFileMatch=2"
        printf ("failedTag=\"No tags found\"\n")
    }
}

function CheckForDupes( field1, field2 )
{
    if (tags[field1] == 1) 
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Duplicate tag found (" field1 "=" field2 ")\"")
    }
    tags[field1]=1
    return 0
}

function MatchSysctlTag( argSysctlRes, field1, field2 )
{

    offset = match( field1, "expand" )
    if ( offset == 0) 
    {
       offset = match( field1, "board" )
    }

    if ( offset != 0 ) 
    {
       ++matchTagsRecognized
        CheckForDupes( field1, field2 )
        
        #the tag is a "match.sysexpand"
        len = offset + RLENGTH
        tag=substr( field1, len )
        
        #find the matching tag in the sysinfo results
        pattern = field2

        offset = match( argSysctlRes, pattern )

        if ( offset != 0 )
        {
            return 1
        }
        else
        {
            failedTag = field1 "=" field2
            return 2
        }
    }
    return 0
}

#____________________________________________________________________
# This script attempts to (as quickly as possible) determine whether
# the specified config file matches the current card (as defined by the 
# collection of devices, pci select info, etc.).
#
# Input parameters are the number of devices and several arrays, passed in as
# newline separated records:  
#   argSysCtl - output of "sysctl -a | grep 14k"
#   argFileName - Name of the file being parsed
#
# Output parameters:
# It sets the variable crdCfgFileMatched to 1 if the specified file was
# matched, 2 if not, and 0 if an error occured in the script.
# If it fails, the tag that failed to match is written to failedTag
#____________________________________________________________________
{
    #if its not a comment line (does not start with #) (regex to positively match: ^[ \t]*[^#].*)
    if (match ($1, /^[ \t]*#/) == 0)
    {
        #Normalize the fields.  We expect a tag containing no '=',
        # followed by any value string.  Since the value could contain 
        # an '=' and we have to check that there are only 2 fields
        # found in this record, and if there are more, concatenate
        # $2 with the additional ones (inserting a '=')
        if (length ($1) != 0) 
        {
            if (length ($2) == 0) 
            {
                system ("logger -p local0.error \"" logPrefix "(" argFileName ") Tag missing value (" $1 ")\"")
                system ("logger -p local0.error \"" logPrefix "(" argFileName ") Skipping file\"")
                matched = 2
                exit
            }
        }
        field2 = $2
        if ( NF > 2 )
        {
            for (i=3; i<=NF; ++i)
            {
               field2 = field2 "=" $i
            }
        }
        #If it's a match tag
        if ( match( $1, /^[ \t]*match\./ ) ) 
        {
            matchedTag = MatchSysctlTag(argSysctl, $1, field2 )      
            if ( matchedTag == 2 )
            {
                matched = 0
                exit
            }
            if (matchedTag == 0) \
            {
                system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Unrecognized tag found (" $1 ")\"")
            }
        }
    }
}
