#matchMoboToCfgFile.awk

BEGIN {
    print "mbCfgFileMatch=0"
    matched = 1
    FS = "="
    logPrefix = "mobomap: "
    matchTagsRecognized = 0
    includeFiles=""
}
END {
    
    if (matchTagsRecognized != 0) 
    {
        if ( matched == 1 )
        {
            print "mbCfgFileMatch=1"
            if (getSlots == 1) 
            {
                printf ( "numSlots=\"%s\"\n", numSlots )
                for (i=1; i<=numSlots; ++i) 
                {
                    printf ( "selector[%s]=\"%s\"\n", i, selector[i] )
                }
            }
        }
        else
        {
            print "mbCfgFileMatch=2"
            printf ("failedTag=\"%s\"\n", failedTag)
        }
    }
    else
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") No match tags specified; ignoring file\"")
        print "mbCfgFileMatch=2"
        printf ("failedTag=\"No tags found\"\n")
    }

    printf("mbCfgIncludeFile=\"%s\"\n", includeFiles )
}

function CheckForDupes( field1, field2 )
{
    if (tags[field1] == 1) 
    {
        system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Duplicate tag found (" field1 "=" field2 ")\"")
    }
    tags[field1]=1
    return 0
}

#____________________________________________________________________
#Given the results of running pciconf and a line from the config file
# (fields 1 and 2), this function checks if the line is a pciconf tag
# and matches the value if it is.
# Returns 1 for match, 2 for no failed match, and 0 for tag not found
#____________________________________________________________________
function MatchPciconfTag( argPciconfRes, field1, field2 )
{
    #Found a "match" tag
    offset = match( field1, /^[ \t]*match\.pciconf\./ )
    if ( offset != 0 )
    {
        matchTagsRecognized++
        CheckForDupes( field1, field2 )

        #Found a pciconf match tag
        #the tag is a "match.pciconf"
        len = offset + RLENGTH
        selectorStr = substr( field1, len )
        description = field2
        
        #find the matching pci selector in the pciconf results
        pattern = "[A-z]+[0-9]+@" selectorStr ":"
        offset = match( argPciconfRes, pattern )
        if ( offset != 0 )
        {
            #find the next pci selector (any selector or the end
            start = offset + RLENGTH
            len = match(substr(argPciconfRes, start), /[A-z]+[0-9]+@pci[0-9]+:[0-9]+:[0-9]+/) - 1
            if (len == 0)
            {
                #if no more selectors were found, were at the end
                len = length(argPciconfRes)
            }
	    cfgblock = substr(argPciconfRes, start, len)
	    # Match the description in the device   = line, or in the chip id
	    # if it starts with 0x
	    if (match(cfgblock, "device   = [^\n]*" description "[^\n]*") ||
	        (match(description, "^0x") &&
		 match(cfgblock, "^[^\n]*chip=" description)))
	    {
		return 1
	    }
	    failedTag = field1 "=" field2
	    return 2
        }
        else
        {
            failedTag = field1 "=" field2
            return 2
        }
        return 0
    }

    offset = match( field1, /^[ \t]*match\.pciconfMin\./ )
    if ( offset != 0 )
    {
        matchTagsRecognized++
        CheckForDupes( field1, field2 )

        #Found a pciconfMin match tag...these are pciconf tags whose
        # bus number must be at least the number specified, but can be more.
        # we will only check up to 10 bus numbers higher (accounts for
        # 10 bridges).
        len = offset + RLENGTH
        selectorStr = substr( field1, len )
        description = field2

        if (split (selectorStr, sels, ":"))
        {
            split (sels[1], sel1, "i")
            for (i=0; i<10; ++i)
            {
                busnum = sel1[2] + i
                #find the matching pci selector in the pciconf results
                pattern = "[A-z]+[0-9]+@pci" busnum ":" sels[2] ":" sels[3] 
                offset = match( argPciconfRes, pattern )
                if ( offset != 0 )
                {
                    #find the next pci selector (any selector or the end
                    start = offset + RLENGTH
                    offset2 = match( substr( argPciconfRes, start ), /[A-z]+[0-9]+@pci[0-9]+:[0-9]+:[0-9]+/ ) 
                    if ( offset2 == 0 )
                    {
                        #if no more selectors were found, were at the end
                        offset2 = length(argPciconfRes)
                    }
                    #in the block of text from the matching selector to
                    # the next selector (or end) look for "device   ="
                    pattern = "device   ="
                    len = offset2
                    offset3 = match( substr( argPciconfRes, start, len), pattern)
                    if ( offset3 != 0 )
                    {
                        netoffset = start + offset3
                        len = start + offset2 - offset3
                        #starting at the "device   =", look for a matching
                        # description.  If none, return 0.
                        descMatchOffset = match ( substr (argPciconfRes, netoffset, len), description )
                        if ( descMatchOffset != 0 )
                        {
                            return 1
                        }
                    }
                }
            }
        }
        #If we don't find the tag above, we want to fail on it
        failedTag = field1 "=" field2
        return 2
    }

    offset = match( field1, /^[ \t]*match\.pciconfvendor\./ )
    if ( offset != 0 )
    {
        matchTagsRecognized++
        CheckForDupes( field1, field2 )

        #Found a pciconfMin match tag...these are pciconf tags whose
        # bus number must be at least the number specified, but can be more.
        # we will only check up to 10 bus numbers higher (accounts for
        # 10 bridges).
        len = offset + RLENGTH
        selectorStr = substr( field1, len )
        description = field2

        if (split (selectorStr, sels, ":"))
        {
            split (sels[1], sel1, "i")
            for (i=0; i<10; ++i)
            {
                busnum = sel1[2] + i
                #find the matching pci selector in the pciconf results
                pattern = "[A-z]+[0-9]+@pci" busnum ":" sels[2] ":" sels[3] 
                offset = match( argPciconfRes, pattern )
                if ( offset != 0 )
                {
                    #find the next pci selector (any selector or the end
                    start = offset + RLENGTH
                    offset2 = match( substr( argPciconfRes, start ), /[A-z]+[0-9]+@pci[0-9]+:[0-9]+:[0-9]+/ ) 
                    if ( offset2 == 0 )
                    {
                        #if no more selectors were found, were at the end
                        offset2 = length(argPciconfRes)
                    }
                    #in the block of text from the matching selector to
                    # the next selector (or end) look for "device   ="
                    pattern = "vendor   ="
                    len = offset2
                    offset3 = match( substr( argPciconfRes, start, len), pattern)
                    if ( offset3 != 0 )
                    {
                        netoffset = start + offset3
                        len = start + offset2 - offset3
                        #starting at the "device   =", look for a matching
                        # description.  If none, return 0.
                        descMatchOffset = match ( substr (argPciconfRes, netoffset, len), description )
                        if ( descMatchOffset != 0 )
                        {
                            return 1
                        }
                    }
                }
            }
        }
        #If we don't find the tag above, we want to fail on it
        failedTag = field1 "=" field2
        return 2
    }
    return 0
}
#____________________________________________________________________
#Given the results of running sysinfo and a line from the config file
# (fields 1 and 2), this function checks if the line is a sysinfo tag
# and matches the value if it is.
# Returns 1 for match, 2 for no failed match, and 0 for tag not found
#____________________________________________________________________
function MatchSysinfoTag( argSysinfoRes, field1, field2 )
{
    offset = match( field1, /^[ \t]*match\.sysinfo\./ )
    if ( offset != 0 ) 
    {
        ++matchTagsRecognized
        CheckForDupes( field1, field2 )

        #the tag is a "match.sysinfo"
        len = offset + RLENGTH
        tag=substr( field1, len )
        val=field2
        
        #find the matching tag in the sysinfo results
        pattern = tag
        offset = match( argSysinfoRes, pattern )
        if ( offset != 0 )
        {
            #find the end of line  (dont include = )
            start = offset + RLENGTH + 1
            offset2 = match( substr( argSysinfoRes, start ), /\n/ ) 
            if ( offset2 == 0 )
            {
                #if no more newlines were found, were at the end
                offset2 = length(argSysinfoRes)
            }
            #in the block of text from the matching tag to
            # the next tag (or end) look for the tag value
            pattern = val
	    # offset2 (\n pos) - 1 so its not included
            len = offset2 - 1
            if (substr( argSysinfoRes, start, len) == pattern)
            {
                return 1
            }
            else
            {
                failedTag = field1 "=" field2
                return 2
            }
        }
        else
        {
            failedTag = field1 "=" field2
            return 2
        }
    }
    return 0
}
#____________________________________________________________________
#Given the results of running dmesg and a line from the config file
# (fields 1 and 2), this function checks if the line is a dmesg tag
# and matches the value if it is.
# Returns 1 for match, 2 for no failed match, and 0 for tag not found
#____________________________________________________________________
function MatchDmesgTag( argDmesgRes, field1, field2 )
{
    offset = match( field1, /^[ \t]*match\.dmesg\./ )
    if ( offset != 0 ) 
    {
        ++matchTagsRecognized
        CheckForDupes( field1, field2 )
        
        #the tag is a "match.dmesg"
        len = offset + RLENGTH
        tag=substr( field1, len )
        
        #find the matching tag in the sysinfo results
        pattern = field2
        offset = match( argDmesgRes, pattern )
        if ( offset != 0 )
        {
            return 1
        }
        else
        {
            failedTag = field1 "=" field2
            return 2
        }
    }
    return 0
}
#____________________________________________________________________
# Main body of script
# This script attempts to (as quickly as possible) determine whether
# the specified config file matches the current motherboard.
# Inputs:
#   getSlots - 1 if we are retesting the motherboard and want to know how
#              many slots are available on the board and their PCI selectors,
#              0 otherwise. (optional)
#   argFileName - Name of the file being parsed
#
# Outputs:
#   mbCfgFileMatched - set to 1 if the specified file was matched, 2 if not,
#                      and 0 if an error occured in the script.
#   numSlots - set to the value of the mobo.pciexp.count tag if getSlots was
#              specified as 1.  Unset otherwise.
#   selectors - array of pciSelectors used to access each of the slots. Unset
#               if getSlots is not set
#____________________________________________________________________
{
    if (match ( $1, /^[ \t]*include/) )
    {
        # include file.
        includeFiles=includeFiles " " $2
    }
    else if (match ($1, /^[ \t]*#/) == 0)
    {
        #if its not a comment line (does not start with #) (regex to positively match: ^[ \t]*[^#].*)
        #Normalize the fields.  We expect a tag containing no '=',
        # followed by any value string.  Since the value could contain 
        # an '=' and we have to check that there are only 2 fields
        # found in this record, and if there are more, concatenate
        # $2 with the additional ones (inserting a '=')
        if (length ($1) != 0) 
        {
            if (length ($2) == 0) 
            {
                system ("logger -p local0.error \"" logPrefix "(" argFileName ") Tag missing value (" $1 ")\"")
                system ("logger -p local0.error \"" logPrefix "(" argFileName ") Skipping file\"")
                matched = 2
                exit
            }
        }
        field2 = $2
        if ( NF > 2 )
        {
            for (i=3; i<=NF; ++i)
            {
               field2 = field2 "=" $i
            }
        }
        
        #convert " to \"
        pattern = "\""
        if (match (field2, pattern))
        {
            str1 = substr( field2, 1, RSTART-1 )
            str2 = substr( field2, RSTART+1 )
            field2 = str1 "\\\""
            while (match (str2, pattern))
            {
                str1 = substr( str2, 1, RSTART-1 )
                str2 = substr( str2, RSTART+1 )
                field2 = field2 str1 "\\\""
            }
            field2 = field2 str2 
        }
	# remove "
        if (match (argSysinfoRes, pattern))
        {
            str1 = substr( argSysinfoRes, 1, RSTART-1 )
            str2 = substr( argSysinfoRes, RSTART+1 )
            argSysinfoRes = str1 
            while (match (str2, pattern))
            {
                str1 = substr( str2, 1, RSTART-1 )
                str2 = substr( str2, RSTART+1 )
                argSysinfoRes = argSysinfoRes str1
            }
            field2 = field2 str2 
	}
        #strip off the trailing newline
        if (match (field2, /\n$/)) 
        {
            len = length ( field2 ) - 1
            field2 = substr( field2, 1, len )
        }

        if ( match( $1, /^[ \t]*match\./ ) ) 
        {
            matchedTag = MatchPciconfTag( argPciconfRes, $1, field2 )            
            if ( matchedTag == 2 )
            {
                matched = 0
                exit
            }
            if ( matchedTag == 0 ) 
            {
                #If the last tag match return 0 (indicating that
                # this match tag has not been tested yet) then see
                # if it matches a sysinfo tag.
                matchedTag = MatchSysinfoTag( argSysinfoRes, $1, field2 )
                if ( matchedTag == 2 )
                {
                    #didnt find a match
                    matched = 0
                    exit
                }
            }
            if ( matchedTag == 0 ) 
            {
                #Still have not successfully tested the value of this tag.
                # Check if it is a dmesg tag
                matchedTag = MatchDmesgTag( argDmesgRes, $1, field2 )
                if ( matchedTag == 2 )
                {
                    #didnt find a match
                    matched = 0
                    exit
                }
            }
            if ( matchedTag == 0) 
            {
                system ("logger -p local0.warning \"" logPrefix "(" argFileName ") Unrecognized tag found (" $1 ")\"")
            }
        }
        else if( getSlots == 1 )
        {
            if ( match( $1, /^[ \t]*mobo\.pciexp\.count/ ) ) 
            {
                numSlots = $2
            }
            if ( match( $1, /^[ \t]*mobo\.pciexp\.slot/ ) ) 
            {
                offset = RLENGTH + RSTART
                match ( substr ($1, offset), /^[0-9]+/ )
                slot = substr ( $1, offset, RLENGTH )
                selector[slot] = $2
            }
        }
    }
}
