#!/bin/sh
#
# if called with an argument, will build the signature file for this
# motherboard as maps/$1
#
if [ -z $install_base ]
then
    install_base=$(dirname $0)
    BASE=$install_base/etc/mobomap
fi
if [ -d /etc/mobomap ]
then
    BASE=/etc/mobomap
    install_base=
fi
if [ -d /usr/local/sandvine/etc ]
then
    BASE=/usr/local/sandvine/etc/mobomap
    install_base=/usr/local/sandvine
fi
OUR_MAP=$(pciconf -l |sed -e 's?.*@??g' -e 's?:.*chip=? ?g' -e 's? rev.*??g')
if [ $# != 0 ]
then
    if [ -f $BASE/maps/$1 ]
    then
	echo "Error: maps/$1 exists"
	exit 1
    fi
    echo "$OUR_MAP" > $BASE/maps/$1
    cp $BASE/maps/UNKNOWN.map $BASE/maps/$1.map
    echo "You now need to edit $BASE/maps/$1.map, and check into clearcase"
    exit 0
fi
MOBO=UNKNOWN
. $BASE/maps/$MOBO.map
for i in $BASE/maps/*
do
    POT_MAP=$(cat $i)
    if [ "$OUR_MAP" = "$POT_MAP" ]
    then
	MOBO=$(basename $i)
    fi
done
rm -f $install_base/etc/motherboard.map
ln -s $BASE/maps/$MOBO.map $install_base/etc/motherboard.map
. $install_base/etc/motherboard.map

# check to see if BIOS can have the TCO_SMI interrupt enabled
# MIN_BIOS_DATE_SMI_NMI and CUR_BIOS_DATE is in the ISO format of YYYYMMDD
# Both args are sourced in from motherboard.map
# 
# NOTE: DO NOT read the motherboard.map file, only source it.
if [ ! -z "$MIN_BIOS_DATE_SMI_NMI" -a ! -z "$CUR_BIOS_DATE" ]
then
    # now parse out, month, day and year to compare with minimum
    # dates and current bios dates.
    min_bios_month=$(echo $MIN_BIOS_DATE_SMI_NMI | awk '{m=substr($1,5,2); print m}')
    min_bios_day=$(echo $MIN_BIOS_DATE_SMI_NMI | awk '{d=substr($1,7,2); print d}')
    min_bios_year=$(echo $MIN_BIOS_DATE_SMI_NMI | awk '{y=substr($1,1,4); print y}')
    #echo "month=$min_bios_month, day=$min_bios_day, year=$min_bios_year"
    cur_bios_month=$(echo $CUR_BIOS_DATE | awk '{m=substr($1,5,2); print m}')
    cur_bios_day=$(echo $CUR_BIOS_DATE | awk '{d=substr($1,7,2); print d}')
    cur_bios_year=$(echo $CUR_BIOS_DATE | awk '{y=substr($1,1,4); print y}')
    #echo "curM=$cur_bios_month, curD=$cur_bios_day, curY=$cur_bios_year"

    validBios=0

    if [ $cur_bios_year -eq $min_bios_year ]
    then
        if [ $cur_bios_month -eq $min_bios_month ]
        then
            if [ $cur_bios_day -ge $min_bios_day ]
            then
                validBios=1
            else
                #echo "day is less than $min_bios_day"
            fi
        else
            if [ $cur_bios_month -gt $min_bios_month ]
            then
                validBios=1
            else
                #echo "month is less than $min_bios_month"
            fi
        fi
    else
        if [ $cur_bios_year -gt $min_bios_year ]
        then
            validBios=1
        else
            #echo "year is less than $min_bios_year"
        fi
    fi

    if [ $validBios -eq 1 ]
    then
        #echo "We have a valid bios!!! enabling TCO_SMI interrupt"
        sysctl hw.watchdog.tco_watchdog_enable_tco_smi_int=1
    fi
fi

logger -s "motherboard-type: $MOBO ($MOTHERBOARD)"

