#parseAndGenerateRcConfLocal.awk
#
#This script parses the existing rc.conf.local file and if the comment is found
# that permits us to modify the file, will set the variable "output". 
# It is up to the caller to dump this to the file.  If the comment
# is not found, the script does not fully set the variable (though it may not
# be empty).  In either case, "ret" is set.

#Field separator is "\n"
#Start by setting the return var to 0. This way if there's an error in the
# script and we don't end up running our END section, the caller will know.
BEGIN {
    print "ret=0"
    matchState = 0
    FS = "\n"
    output = ""
    unknown = ""
    firstTime = 1
    lastLineWasBlank = 0
    ifConfigsFoundInState0 = 0
}
#Set our output variables
END {
    if (matchState == 3) 
    {
        #We never hit another tag, dump what we have
        output = output cfgBlock "\n" unknown "\n" 
    }
    else if (matchState == 0) 
    {
        #If we didn't find the magic comment, or any ifconfig_ lines, we can
        # dump in our config block
        if (ifConfigsFoundInState0 == 0) 
        {
            output = output cfgBlock
        }
    }
    if ( ret == 2 )
    {
        print "ret=2"
    }
    else
    {
        print "ret=1"
    }
    printf ("output=\"%s\"\n", output)
    printf ("debug1=\"%s\"\n", debug1)
    printf ("debug2=\"%s\"\n", debug2)
    printf ("debug3=\"%s\"\n", debug3)
    printf ("debug4=\"%s\"\n", debug4)
}
#____________________________________________________________________
# Input parameters:
#   comment1 - the 1st line of our magic comment 
#   comment2 - the 2nd line of our magic comment 
#   comment3 - the 3rd line of our magic comment
#   cfgBlock - the configuration block that we need to dump in
#
# Output parameters:
#   ret - 0 if error, 1 if "output" is populated (overwrite rc.conf.local)
#         2 if "output" is empty (do not overwrite rc.conf.local)
#   output - Text of new rc.conf.local file.
#____________________________________________________________________
{
    #make sure quotes get transferred properly
    gsub( "\"", "\\\"", $0 )

    if (firstTime) 
    {
        firstTime = 0
        str = cfgBlock
        newblock = ""
        offset = 1
        while ( match( str, "_quote_") )
        {
            start = RSTART + offset -1
            str1 = substr( cfgBlock, offset, start-offset ) 
            newblock = newblock str1 "\\\""
            offset = start+RLENGTH
            str = substr( cfgBlock, offset )
        }
        newblock = newblock str
        cfgBlock = newblock
    }
    #match state 0 is before we've matched any of the comment lines
    #match state 1 is after we've matched the first comment line. If the next
    #  line is not comment 2, we're done
    #match state 2 is after we've matched the first two comment lines. If the
    #  next line is not comment 3, we're done
    #match state 3 is after we've matched all three comment lines.  We stay 
    #  in this state until we find a line that is none of "ifconfig_", blank, 
    #  or a comment line followed by an "ifconf_" (we want to make sure that
    #  comment lines preceding a different type of tag get transferred 
    #  correctly)
    #match state 4 is after we've left 3 and we're just copying the rest of the
    #  file over verbatim (much like 0, except we're not looking for magic
    #  comments
    if (matchState == 0)
    {
        if (match ($1, comment1) )
        {
            matchState = 1
        }
        else
        {
            if (match ($1, /^[ \t]*ifconfig_/) )
            {
                ifConfigsFoundInState0 = 1
            }
            output = output $1 "\n"
        }
    }
    else if (matchState == 1)
    {
        if (match ($1, comment2) )
        {
            matchState = 2
        }
        else
        {
            ret = 2
            exit
        }
    }
    else if (matchState == 2)
    {
        if (match ($1, comment3) )
        {
            matchState = 3
        }
        else
        {
            ret = 2
            exit
        }
    }
    else if (matchState == 3)
    {
        if (match ($1, /^[ \t]*\#/) )
        {
            #it's a comment
            unknown = unknown $1 "\n"
            lastLineWasBlank = 0
        }
        else if (match ($1, /^[ \t]+$/)) 
        {
            #it's a whitespace line
            if ( lastLineWasBlank == 0 )
            {
                unknown = unknown $1 "\n"
                lastLineWasBlank = 1
            }
        }
        else if (match ($1, /^$/)) 
        {
            #it's a blank line
            if ( lastLineWasBlank == 0 )
            {
                unknown = unknown $1 "\n"
                lastLineWasBlank = 1
            }
        }
        else if (match ($1, /^[ \t]*ifconfig_/))
        {
            #it's another ifconfig line
            #wipe anything we've been saving in case we hit a different tag
            unknown = ""
            lastLineWasBlank = 0
        }
        else
        {
            debug1 = unknown
            #it's another tag (not ifconfig_), so let's append our cfgBlock, 
            # anything we've stored in our unknown buffer, and the current
            # line to the output and switch to state 4 (verbatim copy)
            output = output cfgBlock "\n" unknown "\n" $1 "\n"
            matchState = 4
            unknown = ""
            lastLineWasBlank = 0
        }
    }
    else if (matchState == 4)
    {
        if (match ($1, /^[ \t]+$/)) 
        {
            #it's a whitespace line
            if ( lastLineWasBlank == 0 )
            {
                output = output $1 "\n"
                lastLineWasBlank = 1
            }
        }
        else if (match ($1, /^$/)) 
        {
            #it's a blank line
            if ( lastLineWasBlank == 0 )
            {
                output = output $1 "\n"
                lastLineWasBlank = 1
            }
        }
        else
        {
            output = output $1 "\n"
            lastLineWasBlank = 0
        }
    }
}
